/* mods 871211, 24 for HyperCard XFCN work -- remove <stdio> stuff and
 * go to the Mac toolbox and HC system ... ^z
 *
 * REMOVE the static declarations -- for safety's sake in XFCN!
 */

/* This is the file 'bufio.c' ... by ^z, 870830-0913- ... it includes 
 * definitions of some buffer words to accumulate information on its
 * way to/from the disks ... use these to speed up operations and reduce
 * disk head movements, in place of calls to fputc(), fread(), fwrite(),
 * etc. ... try to make them very general so that they will simplify
 * other tasks....
 */

#include "buildIndex.1.h"


/* function to create a zbuffer and set its parameters appropriately
 */

void create_zbuffer (n, bufsize, buffile, bufitemsize, zbuffer)
  int n, bufitemsize;
  long bufsize;
  int buffile;
  ZBUF zbuffer[];
  {
	zbuffer[n].zbufbase = make_buf (bufsize);
	zbuffer[n].zbufp = zbuffer[n].zbufbase;
	zbuffer[n].zbufcounter = 0;
	zbuffer[n].zbufsize = bufsize;
	zbuffer[n].zbuffilep = buffile;
	zbuffer[n].zbufitemsize = bufitemsize;
  }


/* function to return a pointer to the next item in a chosen input
 * buffer 'n'; it reloads the buffer if necessary ... returns NULL
 * pointer when there's nothing left in the file ...
 */

char *next_input_item (n, zbuffer)
  int n;
  ZBUF zbuffer[];
  {
	char *result;
	
	if (zbuffer[n].zbufcounter == 0)
		load_zbuffer (n, zbuffer);
	
	zbuffer[n].zbufcounter -= zbuffer[n].zbufitemsize;
	if (zbuffer[n].zbufcounter >= 0)
	  {
		result = zbuffer[n].zbufp;
		zbuffer[n].zbufp += zbuffer[n].zbufitemsize;
		return (result);
	  }
	else
		return (NULL);
  }


/* function to load the nth zbuffer appropriately ... it resets the buffer
 * pointers, etc. ... 
 */

void load_zbuffer (n, zbuffer)
  int n;
  ZBUF zbuffer[];
  {
	long nread;
	OSErr err;

	nread = zbuffer[n].zbufsize;
	err = FSRead (zbuffer[n].zbuffilep, &nread, zbuffer[n].zbufbase);

	zbuffer[n].zbufp = zbuffer[n].zbufbase;
	zbuffer[n].zbufcounter = nread;
	
	if (err != noErr && err != eofErr)
	  {
		give_msg ("\pSorry, fatal error loading a buffer -- click mouse to exit, then reboot!");
		beepWait ();
	  	RestoreA4();
		ExitToShell ();
	  }
	
	return;
  }


/* function to return a pointer to the right place to store the next
 * output item for the nth zbuffer ... when the buffer becomes full,
 * it flushes it to disk, resets pointers, etc.
 */

char *next_output_item (n, zbuffer)
  int n;
  ZBUF zbuffer[];
  {
	char *result;

	if (zbuffer[n].zbufcounter == zbuffer[n].zbufsize)
		flush_zbuffer (n, zbuffer);
	
	result = zbuffer[n].zbufp;
	zbuffer[n].zbufp += zbuffer[n].zbufitemsize;
	zbuffer[n].zbufcounter += zbuffer[n].zbufitemsize;
	return (result);
  }


/* function to flush out a zbuffer to the disk ... 
 */

void flush_zbuffer (n, zbuffer)
  int n;
  ZBUF zbuffer[];
  {
	long chars_written;
	
	if (zbuffer[n].zbufcounter == 0)
		return;

	chars_written = zbuffer[n].zbufcounter;
	if (FSWrite (zbuffer[n].zbuffilep, &chars_written,
				zbuffer[n].zbufbase) != noErr || chars_written == 0)
	  {
		give_msg ("\pSorry, fatal error flushing a buffer -- click mouse to exit, then reboot!");
		beepWait ();
	  	RestoreA4();
		ExitToShell ();
	  }
	
	zbuffer[n].zbufp = zbuffer[n].zbufbase;
	zbuffer[n].zbufcounter = 0;
  }
  

