/* headers for my buildIndex XFCN project ... mostly prototypes and
 * data structures, etc. ... adapted from MultIndexer header file....
 *
 * 871211 ^z
 */

/* files to include for the XFCN work...
 */

#include <MacTypes.h>
#include <OSUtil.h>
#include <FileMgr.h>
#include <WindowMgr.h>
#include <StdFilePkg.h>
#include <HyperXCmd.h>

/* sort on KEY_LENGTH characters ... make it 28 rather arbitrarily as an
 * experiment ... want it long enough to avoid truncation of legitimate
 * words, but short enough to save some space in the *.k files ... an
 * alternative value is 12, for compatibility with the older ndxr.c program
 * and brwsr.c, if they haven't been revised to allow for longer keys....
 */
#define KEY_LENGTH  28

/* define a structure for the index_keys file
 */
typedef struct
  {
	char kkey[KEY_LENGTH];
	long ccount;
  }  KEY_REC;

/* define a structure for my simpleminded I/O buffers ...
 */
typedef struct
  {
	char *zbufbase;
	char *zbufp;
	long zbufcounter;
	long zbufsize;
	int  zbuffilep;
	int  zbufitemsize;
  }  ZBUF;


/* merge this many files at each stage of the merging operation for index
 * building ... 2 means a binary merge, etc. ... one needs to have at least
 * 5 + 2 * NMERGE I/O buffers around:  for each of NMERGE files, there is
 * a *.k keys file and a *.p pointers file; plus there must be a single
 * output *.k and a single output *.p file; plus there is the need for stdin,
 * stdout, and stderr to be open as well.  Thus, I have found that a 4-way
 * merge (NMERGE = 4) works pretty nicely....
 */
#define NMERGE 4

#ifndef TRUE
#define TRUE   1
#endif

#ifndef FALSE
#define FALSE  0
#endif

#ifndef NULL 
#define NULL   0
#endif

#ifndef EOF
#define EOF  (-1)
#endif


/* define these to allow use of global variables in my XFCNs
 */

#define SetUpA4()		asm  {	move.l a4,-(sp)		\
								move.l a0,a4	}

#define RestoreA4()		asm  {	move.l (sp)+,a4	}


/* put the prototypes for my functions here... 
 */


/* in buildIndex.c main program */
pascal void main (XCmdBlockPtr paramPtr);
int GetFileZ (Str255 *fn, int *vRef, int x0, int y0);
void pStrCopy (char *p1, char *p2);
long atol (char *s);
int putNum (char *ans, long num);
void give_msg (char *msg);
void beepWait (void);

/* in bufio.c */
void create_zbuffer (int n, long bufsize, int buffile, int bufitemsize,
						ZBUF zbuffer[]);
char *next_input_item (int n, ZBUF zbuffer[]);
void load_zbuffer (int n, ZBUF zbuffer[]);
char *next_output_item (int n, ZBUF zbuffer[]);
void flush_zbuffer (int n, ZBUF zbuffer[]);

/* in build_indices.c */
int build_indices (long zbufsiz, int doc_file, int vRef0, int pass_number);
void init_filter_table (void);

/* in doc_buf.c */
char *make_buf (long bufsiz);
long load_doc_buffer (char *doc, long doc_bufsiz, char **ptr, int doc_file);
int zgetc (int refNum);
char *strncpy(char *s1, char *s2, int n);

/* in merge_files.c */
void nway_merge_kpfiles (int ink[], int inp[], int outk, int outp, int n,
							long zbufsiz);
void copy_ptr_recs (int inzbuf, long count, int outzbuf, ZBUF zbuffer[]);
void copy_key_rec (char *kkey, long ccount, int outzbuf, ZBUF zbuffer[]);
int merge_not_finished (KEY_REC *kr[], int n);
int smallest_key (KEY_REC *kr[], int n);

/* in merge_indices.c */
int merge_indices (long zbufsiz, int doc_file, int vRef0,
			int file_number, int generation_number, Str255 doc_filename);

/* in file_utils.c */
long write_sorted_files (char *doc, char **ptr, long nwords, long offset,
		long zbufsiz, int pass_number, int vRef0);
int is_new_word (char *w0, char *w1);
void write_new_key (char *p, char *kp);
long file_size (int refnum);

/* in merge_utils.c */
int open_inkfile (int file_num, int vRef0, int generation_number);
int open_inpfile (int file_num, int vRef0, int generation_number);
void fix_oddball_file_name (int vRef0, int generation_number,
			int file_number);
void fix_final_file_names (Str255 doc_filename, int vRef0,
			int generation_number);
int open_outkfile (int vRef0, int generation_number, int file_number);
int open_outpfile (int vRef0, int generation_number, int file_number);
void remove_used_infiles (int n, int vRef0, int generation_number,
			int file_number);
void close_infiles (int ink[], int inp[], int n);

/* in open_files.c */
int open_kfile (int vRef0, int pass_number);
int open_pfile (int vRef0, int pass_number);
long zftell (int refnum);

/* in zqsort.c */
void zqsort (char **first, char **last);
int compare_ptrs (char **p1, char **p2);
int zstrcmp (char *s1, char *s2);

