/* mods 871211,22 for HyperCard XFCN work -- remove <stdio> stuff and
 * go to the Mac toolbox and HC system ... ^z
 *
 * some old documentation retained for completeness....
 */

/* file build_indices.c ... by ^z, 870820-0913-...
 *
 * contains subroutine to build indices for each chunk of input document
 * (database) text file for program qndxr ... 
 *
 * Strategy is as outlined in qndxr:  take in a big chunk of the doc_file,
 *	generate the pointers to each word in the buffer as the buffer contents
 *	are converted into appropriate (all caps, delimiters filtered into
 *	spaces) form for sorting; sort the pointers in memory; and then write
 *	out to disk the pointers and keys in the ndxr.c format for *.k and *.p
 *	files.
 *
 * Allocate space for the doc and ptr buffers here so as to maximize use
 * of available memory ... note that we need to have room for the doc
 * buffer, for a ptr buffer that might (in the WORST case of a file full
 * of 1-letter words!) be twice as long as the doc buffer, and also space
 * for two standard zbuffers to accumulate the output *.k and *.p file
 * info in before sending it to disk.
 *
 * Note that for speed, while they are being sorted the pointers just point
 * 	directly to the key strings in the input buffer; they must be converted
 *	into true offset pointers relative to the 0th byte of the document file
 *	as they are written to disk in the *.p file!  Make sure that all of
 *	the delimiters in the document/database buffer are converted into
 *	'\0' characters so that string comparison functions will work right.
 *
 * Also note that to avoid edge effects at the end of the buffer, an extra
 *	amount of space is required at the end of the buffer, of length
 *	KEY_LENGTH, to accomodate the end of the last word in the buffer.
 */

#include "buildIndex.1.h"

extern char filter_table[];

int build_indices (zbufsiz, doc_file, vRef0, pass_number)
  long zbufsiz;
  int doc_file, vRef0, pass_number;
  {
	long doc_bufsiz, offset, nwords, ndistinctwords;
	char *doc, **ptr, info[128];
	int i;

	doc_bufsiz = 2 * NMERGE * zbufsiz / 3 - KEY_LENGTH;
	doc = make_buf (doc_bufsiz + KEY_LENGTH);
	
	ptr = (char **)make_buf (doc_bufsiz * 2);

	offset = zftell (doc_file);	
	
	nwords = load_doc_buffer (doc, doc_bufsiz, ptr, doc_file);

	if (nwords == 0)
	  {
		DisposPtr (doc);
		DisposPtr (ptr);
		return (FALSE);
	  }
	
	zqsort (ptr, ptr + nwords);

	ndistinctwords = write_sorted_files (doc, ptr, nwords, offset, zbufsiz,
							pass_number, vRef0);
	
	DisposPtr (doc);
	DisposPtr (ptr);

	strncpy (info + 1, "Index subfile #", 15);
	i = 16 + putNum (info + 16, pass_number + 1);
	strncpy (info + i, ":  ", 3);
	i += 3;
	i += putNum (info + i, nwords);
	strncpy (info + i, " total words, ", 14);
	i += 14;
	i += putNum (info + i, ndistinctwords);
	strncpy (info + i, " distinct words.", 16);
	i += 16;
	info[0] = i - 1;
	give_msg (info);
	
	return (TRUE);
  }


