/* mods 871211 for HyperCard XFCN work -- remove <stdio> stuff and
 * go to the Mac toolbox and HC system ... ^z
 */

/* file doc_buf.c ...  by ^z 870830-0919-...
 * functions to load in text from the document file and then
 * save out keys and pointers to the *.k and *.p files ...
 * modified 871007-... to unify the doc-buffer loading with the
 * character-filtering and the pointer array building....
 */


#include "buildIndex.1.h"
extern char filter_table[];

/* function to create a buffer for me to use...
 */

char *make_buf (bufsiz)
  long bufsiz;
  {
	char *buf;
	
	buf = NewPtr (bufsiz);

	if (buf == 0)
	  {
		give_msg ("\pSorry, fatal error creating a buffer -- click mouse to exit, then reboot!");
		beepWait ();
	  	RestoreA4();
		ExitToShell ();
	  }
	
	return (buf);
  }


/* function to load the document buffer ... bring in doc_bufsiz
 * characters, and then enough more to finish out the final word,
 * followed by a terminal delimiter .... as the characters are scanned
 * filter them appropriately (change delimiters to '\0's) and
 * build the pointer array in memory to the first character of each
 * word ... return the total number of words that were
 * read in to the buffer (zero if we're finished with the file)
 *
 * ... note that one must be sure to pull in and throw away
 * any excess characters beyond KEY_LENGTH in the final word, so that
 * the remaining fragment doesn't show up as the first "word" in the
 * next chunk of the file....
 *
 * For maximum speed and efficiency, use a hybrid approach:  do a
 * big buffer-sized chunk of the file first, then take one character at
 * a time to finish off....
 */
 
long load_doc_buffer (doc, doc_bufsiz, ptr, doc_file)
  register char *doc, **ptr;
  long doc_bufsiz;
  int doc_file;
  {
	int  i, err;
	register int c, in_a_word = FALSE;
	char **ptr0, *end_doc_buf;
 	
 	ptr0 = ptr;
 	
 	err = FSRead (doc_file, &doc_bufsiz, doc);
 	if (err != noErr && err != eofErr)
	  {
		give_msg ("\pSorry, fatal error reading text -- click mouse to exit, then reboot!");
		beepWait ();
	  	RestoreA4();
		ExitToShell ();
	  }

 	end_doc_buf = doc + doc_bufsiz;
 	
 	while (doc < end_doc_buf)
	  {
		c = filter_table[*(unsigned char *)doc];

		if (c == '\0')
			in_a_word = FALSE;
 		else if (! in_a_word)
		  {
			*ptr++ = doc;
			in_a_word = TRUE;
		  }
 
 		*doc++ = c;
 	  }
 	
 	if (err != eofErr && doc == end_doc_buf && in_a_word)
	  {
  		for (i = 0; i < KEY_LENGTH; ++i)
 		  {
 			c = zgetc (doc_file);
 			if (c == EOF)
 			  {
 				*doc++ = '\0';
 				break;
 			  }
 			c = filter_table[c];
 			if (c == '\0')
 			  {
 				*doc++ = '\0';
 				break;
 			  }
 			*doc++ = c;
 		  }
 		if (i == KEY_LENGTH)
 			while ((c = zgetc (doc_file)) != EOF && filter_table[c])
				;
 	  }
 	
 	return (ptr - ptr0);
  }


/* my function to replace getc() ... give up and call it EOF on any error
 * at all, at this point! ... note also that we must return the UNSIGNED
 * character read in, to handle Mac special characters properly....
 */

int zgetc (refNum)
  int refNum;
  {
  	unsigned char readbuf[1];
  	long count = 1;
  	
	if (FSRead (refNum, &count, readbuf) == noErr)
		return (readbuf[0]);
	else
		return (EOF);
  }


/* function to copy a string of length up to n from s2 to s1.... from
 * LSC library ...
 */

char *strncpy (s1, s2, n)
  register char *s1, *s2;
  register int n;
  {
	char *s0 = s1;
	
	if (n > 0)
	{
		while (n-- && (*s1++ = *s2++));
	
		while (n > 0)
		{	/* pad out to n chars -- H&S specs it this way... */
		
			*s1++ = '\0';
			n--;
		}
	}
		
	return (s0);
}

