/* mods 871211 for HyperCard XFCN work -- remove <stdio> stuff and
 * go to the Mac toolbox and HC system ... ^z
 */

/* file file_utils.c ... by ^z -- 870820-0913-...
 * some utility routines for qndxr project, associated with files...
 */

#include "buildIndex.1.h"

/* function to write out sorted k & p files based on the doc and ptr
 * arrays in memory....
 *
 * The kfile format is as described in detail elsewhere:
 *	the key word, turned into all capital letters and with spaces
 *		afterward, of fixed length KEY_LENGTH; and
 *	the cumulative count of how many words have passed before, including
 *		the current word, a long integer.
 *
 * Function revised 870907-... by ^z to use zbuffer method....
 */

long write_sorted_files (doc, ptr, nwords, offset, zbufsiz, pass_number,
							vRef0)
  char *doc, **ptr;
  long nwords, offset, zbufsiz;
  int pass_number, vRef0;
  {
	int kfile, pfile;
	char *prev_word;
	KEY_REC *outk;
	long *outp, i;
	ZBUF zbuffer[2];
	
	if (nwords == 0)
		return;
	
	kfile = open_kfile (vRef0, pass_number);
	pfile = open_pfile (vRef0, pass_number);
	
	create_zbuffer (0, zbufsiz, kfile, sizeof(KEY_REC), zbuffer);
	create_zbuffer (1, zbufsiz, pfile, sizeof(long), zbuffer);

	prev_word = ptr[0];
	outk = (KEY_REC *)next_output_item (0, zbuffer);
	write_new_key (ptr[0], outk->kkey);
	
	for (i = 0; i < nwords; ++i)
	  {
		if (is_new_word (prev_word, ptr[i]))
		  {
			outk->ccount = i;
			outk = (KEY_REC *)next_output_item (0, zbuffer);
			write_new_key (ptr[i], outk->kkey);
			prev_word = ptr[i];
		  }
		outp = (long *)next_output_item (1, zbuffer);
		*outp = (ptr[i] - doc) + offset;
	  }
	outk->ccount = i;

	flush_zbuffer (0, zbuffer);
	flush_zbuffer (1, zbuffer);
	
	DisposPtr (zbuffer[0].zbufbase);
	DisposPtr (zbuffer[1].zbufbase);
	
	i = file_size (kfile) / sizeof(KEY_REC);
	FSClose (kfile);
	FSClose (pfile);
	
	return (i);
  }


/* function to determine if the current word is the same as or different
 * from the previous word -- if it is different, we'll need to write an
 * entry out to the key file kfile -- compare the words up to the first
 * '\0', or for a maximum distance of KEY_LENGTH, and return TRUE
 * if they differ, FALSE if they are identical that far.  Thus, a simple
 * call to zstrcmp() does the job.... but keep ours as a function instead
 * of a macro call for the moment, for safety and readability....
 */

int is_new_word (w0, w1)
  char *w0, *w1;
  {
	return (zstrcmp (w0, w1));
  }


/* function to write out a new key entry in the key_file:
 * KEY_LENGTH letters consisting of the key word (which will be found
 * delimited by a '\0'), followed by enough blanks to fill out the
 * record to total length KEY_LENGTH ...
 */

void write_new_key (p, kp)
  register char *p, *kp;
  {
	register int i, c;

	for (i = 0; i < KEY_LENGTH; ++i)
	  {
		c = *p++;
		if (c == '\0')
			break;
		*kp++ = c;
	  }

	for ( ; i < KEY_LENGTH; ++i)
		*kp++ = ' ';
	
	return;
  }


/* a very simple function to return the size of a file ... doesn't do any
 * error-checking -- sorry!
 */

long file_size (refnum)
  int refnum;
  {
	long result;
	
	GetEOF (refnum, &result);
	return (result);
  }



