/* routines to merge key & ptr files... 871203-...  ^z
 * adapted from the merging phases of qndxr and multindexer
 * programs...modified 871230 for XFCN use....
 */

#include "mergeIndices.z.h"

extern char *buf[3][2], *bufp[3][2];
extern long bufcount[3][2], first_text_file_size, zbufsiz;
extern int refNum[3][3];


int merge_key_and_ptr_files ()
  {
	int i, j;
	KEY_REC *kr[2], prev_key;
	long prev_cc[2], out_cc;

	give_msg ("\pMerging key and ptr files ... please be patient!");
	prev_cc[FIRSTIN]  = 0;
	prev_cc[SECONDIN] = 0;
	out_cc = 0;
	
	kr[FIRSTIN]  = next_input_key (FIRSTIN);
	kr[SECONDIN] = next_input_key (SECONDIN);
	
	i = smaller_key (kr);
	strncpy (prev_key.kkey, kr[i]->kkey, KEY_LENGTH);
	
	while (merge_not_done (kr))
	  {
	  	i = smaller_key (kr);
	  	if (zstrcmp ((unsigned char *)prev_key.kkey,
	  						(unsigned char *)kr[i]->kkey))
	  	  {
	  	  	output_key_rec (prev_key.kkey, out_cc);
	  	  	strncpy (prev_key.kkey, kr[i]->kkey, KEY_LENGTH);
	  	  }
	  	output_ptr_recs (i, kr[i]->ccount - prev_cc[i]);
	  	out_cc += kr[i]->ccount - prev_cc[i];
	  	prev_cc[i] = kr[i]->ccount;
	  	kr[i] = (KEY_REC *)next_input_key (i);
	  }

	output_key_rec (prev_key.kkey, out_cc);
	give_msg ("\pFinished merging key and ptr files!");
	flush_outbuffer (KEYFILE);
	flush_outbuffer (PTRFILE);
	for (i = 0; i < 3; ++i)
		for (j = 0; j < 2; ++j)
		  {
			DisposPtr (buf[i][j]);
			buf[i][j] = 0;
		  }

	return (1);
  }


/* function to return a pointer to the next key from the input file n ...
 * returns NULL pointer when there's nothing in the file left ... reloads
 * the buffer as necessary .... adapted from next_input_item in qndxr.c ....
 */

KEY_REC *next_input_key (n)
  int n;
  {
	KEY_REC *result;
	
	if (bufcount[n][KEYFILE] == 0)
		load_buffer (n, KEYFILE);
	
	bufcount[n][KEYFILE] -= sizeof (KEY_REC);
	if (bufcount[n][KEYFILE] >= 0)
	  {
	  	result = (KEY_REC *)bufp[n][KEYFILE];
	  	bufp[n][KEYFILE] += sizeof (KEY_REC);
	  	return (result);
	  }
	else
		return (NULL);
  }


/* function to load a buffer with data... reset buffer ptrs, etc. properly...
 * fn = file number (FIRSTIN or SECONDIN) and ft = file type (KEYFILE or
 * PTRFILE) ....
 *
 * note that the buffer length had BETTER be an exact integer multiple
 * of the size of a key and ptr record -- in this implementation, that is,
 * an exact multiple of 32 for 28-letter keys and 4-byte pointers....
 */

void load_buffer (fn, ft)
  int fn, ft;
  {
	long nread;
	OSErr err;
	
	nread = zbufsiz;
	err = FSRead (refNum[fn][ft], &nread, buf[fn][ft]);
	if (err != noErr && err != eofErr)
	  {
		give_msg ("\pFatal error reading data from disk -- click mouse, then reboot!");
		beepWait ();
		RestoreA4();
		ExitToShell();
	  }

	bufp[fn][ft] = buf[fn][ft];
	bufcount[fn][ft] = nread;
	
	return;
  }


/* routine to determine which key of the two being considered for merger
 * is the smaller.... note that a NULL ptr ranks after any other ... also
 * note that in case of a tie, the key from the first file wins over the
 * second file (for stable merging, this is important) ... since a NULL ptr
 * implies that a file has been exhausted, it should never be the case that
 * both of the kr pointers are NULL when this routine is called....
 */
 
int smaller_key (kr)
  KEY_REC *kr[2];
  {
	if (kr[SECONDIN] == NULL)
		return (FIRSTIN);
	if (kr[FIRSTIN] == NULL)
		return (SECONDIN);
	if (zstrcmp ((unsigned char *)kr[FIRSTIN]->kkey,
					(unsigned char *)kr[SECONDIN]->kkey) > 0)
		return (SECONDIN);
	else
		return (FIRSTIN);
  }


/* routine to copy fixed-length keys from one place to another ... from
 * LSC stdio library, with thanks....
 */

char *strncpy (s1, s2, n)
  register char *s1, *s2;
  register int n;
  {
	char *s0 = s1;
	
	if (n > 0)
	  {
		while (n-- && (*s1++ = *s2++))
			;
	
		while (n > 0)
		  {
			*s1++ = '\0';
			n--;
		  }
	  }
		
	return (s0);
  }


/* simple function to see if we are not yet finished with both of the input
 * files coming in to the merge operation ... signified by at least one of
 * the key pointers being non-NULL....
 */

int merge_not_done (kr)
  KEY_REC *kr[];
  {
	if (kr[FIRSTIN] != NULL || kr[SECONDIN] != NULL)
		return (TRUE);
	else
		return (FALSE);
  }


/* my function to compare two strings and give a result as to who is
 * alphabetically earlier.  Note that this is almost the same as strncmp()
 * with the fixed value of KEY_LENGTH as the maximum comparison distance,
 * except that I must be sure to mask the characters to make them positive
 * (since we want to be able to handle the non-ASCII funny letters in
 * the Apple character set properly/consistently).  If the masking isn't
 * done, then inconsistent results can occur with those non-ASCII chars!
 * Use type unsigned char in order to achieve this efficiently....
 */

int zstrcmp (s1, s2)
  register unsigned char *s1, *s2;
  {
	register int n = KEY_LENGTH;
	
	for (; --n && (*s1 == *s2); s1++, s2++)
		if (! *s1)
			break;
		
	return (*s1 - *s2);
  }


/* function to output a key record to the (buffer of) the output file
 * where things are being merged.  Adapted from the similar routine called
 * copy_key_rec in qndxr's merge_files.c collection ....
 */

void output_key_rec (kkey, cc)
  char *kkey;
  long cc;
  {
	KEY_REC *outp;
	
	if (bufcount[MERGEDOUT][KEYFILE] == zbufsiz)
		flush_outbuffer (KEYFILE);
	
	outp = (KEY_REC *)bufp[MERGEDOUT][KEYFILE];
	strncpy (outp->kkey, kkey, KEY_LENGTH);
	outp->ccount = cc;
	bufp[MERGEDOUT][KEYFILE] += sizeof (KEY_REC);
	bufcount[MERGEDOUT][KEYFILE] += sizeof (KEY_REC);
	
	return;
  }


/* routine to send pointer records to the output ... based on the function
 * copy_ptr_recs from qndxr.2 .... moves num ptr records from the file
 * fn ... note that if fn = SECONDIN, it is necessary to add the length
 * of the text file FIRSTIN to the ptr value, to take into account the
 * presence of that file ahead of the second one in the merged text file....
 */

void output_ptr_recs (fn, num)
  int fn;
  register long num;
  {
	register long *inp, *outp;
	
	for ( ; num > 0; --num)
	  {
	  	inp = next_input_ptr (fn);
	  	if (bufcount[MERGEDOUT][PTRFILE] == zbufsiz)
	  		flush_outbuffer (PTRFILE);
	  	outp = (long *)bufp[MERGEDOUT][PTRFILE];
		*outp = *inp;
		if (fn == SECONDIN)
			*outp += first_text_file_size;

		bufp[MERGEDOUT][PTRFILE] += sizeof (long);
		bufcount[MERGEDOUT][PTRFILE] += sizeof (long);
	  }

	return; 	
  }
  

/* routine to get a pointer to the next input ptr in file n ...
 * similar to the next_input_key function.... should never be called
 * more times than there are records to be returned, but in case it
 * is, return NULL ptr ....
 */

long *next_input_ptr (n)
  int n;
  {
	long *result;
	
	if (bufcount[n][PTRFILE] == 0)
		load_buffer (n, PTRFILE);
	
	bufcount[n][PTRFILE] -= sizeof (long);
	if (bufcount[n][PTRFILE] >= 0)
	  {
	  	result = (long *)bufp[n][PTRFILE];
	  	bufp[n][PTRFILE] += sizeof (long);
	  	return (result);
	  }
	else
		return (NULL);
  }


/* routine to flush the output buffer to disk ... ft = KEYFILE or PTRFILE,
 * looks the same....
 */

void flush_outbuffer (ft)
  int ft;
  {
	long chars_written;
	
	if (bufcount[MERGEDOUT][ft] == 0)
		return;
	
	chars_written = bufcount[MERGEDOUT][ft];
	if (FSWrite (refNum[MERGEDOUT][ft], &chars_written, buf[MERGEDOUT][ft])
			!= noErr || chars_written != bufcount[MERGEDOUT][ft])
	  {
		give_msg ("\pFatal error writing data to disk -- click mouse, then reboot!");
		beepWait ();
		RestoreA4();
		ExitToShell();
	  }
	
	bufp[MERGEDOUT][ft] = buf[MERGEDOUT][ft];
	bufcount[MERGEDOUT][ft] = 0;
	
	return;
  }


