/* a HyperCard XFCN that opens the chosen set of index files (text
 * file, *.k keys file, and *.p ptr file) and returns their refNums and
 * file lengths (in bytes) as the result to Hypercard.... beeps and
 * returns null string if failure or if user cancels operations....
 *
 * call it as:     openIndexFiles (x, y)
 *    (where x and y are coordinates (positive) from the top left corner
 * of the screen to determine where to put the files dialog ... may want
 * to use the corner of the Hypercard window on big screens....)
 *
 * get back:       textFileRefNum, textFileLength, keyFileRefNum,
 *                 keyFileLength, ptrFileRefNum, ptrFileLength
 * as a comma-delimited, 0-terminated string.
 *
 * make it XFCN number 2705 and call it "openIndexFiles"
 *
 * Note that *.k and *.p files have type 'CTLZ' .... ^z
 *
 * revisions 871126, 1209, 1223, 1228 ^z
 */


#include <MacTypes.h>
#include <OSUtil.h>
#include <FileMgr.h>
#include <WindowMgr.h>
#include <StdFilePkg.h>
#include <HyperXCmd.h>

pascal void main (XCmdBlockPtr paramPtr);
int GetFileZ (Str255 *fn, int *vRef, long ftype, int x0, int y0);
void pStrCopy (char *p1, char *p2);
int putNum (char *ans, long num);
void complain (XCmdBlockPtr paramPtr, WindowPtr info_window);
long atol (char *s);


pascal void main (paramPtr)
  XCmdBlockPtr paramPtr;
  {
	Str255 fn;
	int vRef, refNumText, refNumKey, refNumPtr, offset, x0, y0;
	long textFileLength, keyFileLength, ptrFileLength;
	WindowRecord w_record;
	WindowPtr info_window;
	Rect b_rect;
	Handle answer;

	if (paramPtr->paramCount != 2)
	  {
		SysBeep (10);
	  	return;
	  }
	 
	x0 = atol (*(paramPtr->params[0]));
	y0 = atol (*(paramPtr->params[1]));
	
	b_rect.top = 30 + y0;
	b_rect.left = 12 + x0;
	b_rect.bottom = 52 + y0;
	b_rect.right = 500 + x0;
	info_window = NewWindow (&w_record, &b_rect, "\p", (Boolean)1, dBoxProc,
		(WindowPtr)-1, (Boolean)0, (long)0);
	ShowWindow (info_window);
	SetPort (info_window);
	TextFont (0);
	b_rect.top = 0;
	b_rect.left = 0;
	b_rect.bottom = 22;
	b_rect.right = 488;

	EraseRect (&b_rect);
	MoveTo (4, 15);
	DrawString ("\pPlease choose a dataspace (indexed text) file to open for browsing...");

	if (! GetFileZ (&fn, &vRef, 'TEXT', x0, y0) ||
				(FSOpen (&fn, vRef, &refNumText) != noErr))
		  {
			complain (paramPtr, info_window);
			return;
		  }
	if (GetEOF (refNumText, &textFileLength) != noErr)
		  {
			complain (paramPtr, info_window);
			return;
		  }

	*(fn + *fn + 1) = '.';
	*(fn + *fn + 2) = 'k';
	*fn += 2;
	if (FSOpen (&fn, vRef, &refNumKey) != noErr)
	  {
	  	SysBeep (10);
		EraseRect (&b_rect);
		MoveTo (4, 15);
		DrawString ("\pCan't find the the key ('.k') file for this dataspace -- please locate it...");
		if (! GetFileZ (&fn, &vRef, 'CTLZ', x0, y0) ||
				(FSOpen (&fn, vRef, &refNumKey) != noErr))
			  {
				complain (paramPtr, info_window);
				return;
			  }
	  }
	if (GetEOF (refNumKey, &keyFileLength) != noErr)
		  {
			complain (paramPtr, info_window);
			return;
		  }
	
	*(fn + *fn) = 'p';
	if (FSOpen (&fn, vRef, &refNumPtr) != noErr)
	  {
	  	SysBeep (10);
		EraseRect (&b_rect);
		MoveTo (4, 15);
		DrawString ("\pCan't find the the ptr ('.p') file for this dataspace -- please locate it...");
		if (! GetFileZ (&fn, &vRef, 'CTLZ', x0, y0) ||
				(FSOpen (&fn, vRef, &refNumPtr) != noErr))
			  {
				complain (paramPtr, info_window);
				return;
			  }
	  }
	if (GetEOF (refNumPtr, &ptrFileLength) != noErr)
		  {
			complain (paramPtr, info_window);
			return;
		  }

	CloseWindow (info_window);
	answer = NewHandle (64);
	offset = putNum (*answer, refNumText);
	*(*answer + offset++) = ',';
 	offset += putNum (*answer + offset, textFileLength);
	*(*answer + offset++) = ',';
	offset += putNum (*answer + offset, refNumKey);
	*(*answer + offset++) = ',';
 	offset += putNum (*answer + offset, keyFileLength);
	*(*answer + offset++) = ',';
	offset += putNum (*answer + offset, refNumPtr);
	*(*answer + offset++) = ',';
 	offset += putNum (*answer + offset, ptrFileLength);
	*(*answer + offset) = '\0';
	paramPtr->returnValue = answer;
	return;
  }


/* following variables and routine do the standard files dialog
 * to get the name of the file to use ... cribbed from the MiniEdit
 * example that comes with LSC....  put the dialog box someplace
 * reasonable ....
 */

int GetFileZ (fnp, vRefp, ftype, x0, y0)
  Str255 *fnp;
  int *vRefp, x0, y0;
  long ftype;
  {
	SFTypeList myTypes;
	Point SFGwhere;
	SFReply reply;

	SFGwhere.v = 90 + y0;
	SFGwhere.h = 82 + x0;
	myTypes[0] = ftype;
	
	SFGetFile (SFGwhere, "\p", 0L, 1, myTypes, 0L, &reply);
	
	if (reply.good)
	  {
		pStrCopy( (char *)reply.fName, (char *)fnp);
		*vRefp = reply.vRefNum;
		return(1);
	  }
	else return(0);
  }


/* routine to copy a pascal string from one place to another.... used in
 * above Standard Files routine.... from LSC library....
 */

void pStrCopy (p1, p2 )
  register char *p1, *p2;
  {
	register int len;
	
	len = *p2++ = *p1++;
	while (--len >= 0)
		*p2++ = *p1++;
	return;
  }


/* function to beep & set the return string to null (= "")
 */

void complain (paramPtr, info_window)
  XCmdBlockPtr paramPtr;
  WindowPtr info_window;
  {
	Handle ans;
	
	SysBeep (10);
  	ans = NewHandle (1);
  	**ans = '\0';
	paramPtr->returnValue = ans;
	CloseWindow (info_window);
	return;
  }

/* function to convert a number into a string and put it into the chosen
 * target place ... returns the number of characters stored ...
 * based on K&R p. 60 example of itoa()....
 */

int putNum (ans, num)
  char *ans;
  long num;
  {
	int i, j, s, result;
	
	i = 0;
	s = 1;
	if (num < 0)
	  {
		num = -num;
		s = -1;
	  }
	
	do
	  	ans[i++] = num % 10 + '0';
	while ((num /= 10) > 0);
	
	if (s < 0)
		ans[i++] = '-';
	result = i;
	
	for (--i, j = 0; j < i; ++j, --i)
	  {
	  	s = ans[i];
	  	ans[i] = ans[j];
	  	ans[j] = s;
	  }

	return (result);
  }
  

/* function to convert alphabetic string to a long integer ... from LSC
 * library.... simplified to avoid using isspace() & isdigit() ....
 */

long atol (s)
  register char *s;
  {
	register char signflag = 0;
	register long r = 0;

	while ((*s == ' '))
		s++;
		
	if (*s == '-')
	  {
		signflag = 1;
		s++;
	  }
	else if (*s == '+')
	 	s++;

	while (*s >= '0' && *s <= '9') 
		r = r * 10 + (*s++ - '0');
	
	return (signflag ? -r : r);
}



